@@ -23,8 +23,12 @@
 #define _HAPROXY_ACTION_T_H
 
 #include <haproxy/applet-t.h>
-#include <haproxy/regex-t.h>
 #include <haproxy/stick_table-t.h>
+#include <haproxy/vars-t.h>
+
+struct session;
+struct stream;
+struct proxy;
 
 enum act_from {
 	ACT_F_TCP_REQ_CON, /* tcp-request connection */
@@ -81,10 +85,6 @@ enum act_name {
 
 	/* common http actions .*/
 	ACT_HTTP_REDIR,
-	ACT_HTTP_SET_NICE,
-	ACT_HTTP_SET_LOGL,
-	ACT_HTTP_SET_TOS,
-	ACT_HTTP_SET_MARK,
 
 	/* http request actions. */
 	ACT_HTTP_REQ_TARPIT,
@@ -166,15 +166,19 @@ struct act_rule {
 		} timeout;
 		struct hlua_rule *hlua_rule;
 		struct {
+			struct list fmt;            /* log-format compatible expression */
 			struct sample_expr *expr;
-			const char *name;
+			uint64_t name_hash;
 			enum vars_scope scope;
+			uint conditions;            /* Bitfield of the conditions passed to this set-var call */
 		} vars;
 		struct {
 			int sc;
+			unsigned int idx;
 		} gpc;
 		struct {
 			int sc;
+			unsigned int idx;
 			long long int value;
 			struct sample_expr *expr;
 		} gpt;
@@ -183,6 +187,10 @@ struct act_rule {
 			void *p[4];
 		} act;                         /* generic pointers to be used by custom actions */
 	} arg;                                 /* arguments used by some actions */
+	struct {
+		char *file;                    /* file name where the rule appears (or NULL) */
+		int line;                      /* line number where the rule appears */
+	} conf;
 };
 
 struct action_kw {
