@@ -422,41 +422,6 @@ static inline uint64_t quic_compute_ack_delay_us(unsigned int time_received,
 	return ((now_ms - time_received) * 1000) >> conn->tx.params.ack_delay_exponent;
 }
 
-/* Initialize a QUIC packet number space.
- * Never fails.
- */
-static inline void quic_pktns_init(struct quic_pktns *pktns)
-{
-	LIST_INIT(&pktns->tx.frms);
-	pktns->tx.next_pn = -1;
-	pktns->tx.pkts = EB_ROOT_UNIQUE;
-	pktns->tx.time_of_last_eliciting = 0;
-	pktns->tx.loss_time = TICK_ETERNITY;
-	pktns->tx.in_flight = 0;
-	pktns->tx.ack_delay = 0;
-
-	pktns->rx.largest_pn = -1;
-	pktns->rx.largest_acked_pn = -1;
-	pktns->rx.arngs.root = EB_ROOT_UNIQUE;
-	pktns->rx.arngs.sz = 0;
-	pktns->rx.arngs.enc_sz = 0;
-	pktns->rx.nb_aepkts_since_last_ack = 0;
-	pktns->rx.largest_time_received = 0;
-
-	pktns->flags = 0;
-}
-
-/* Returns the current largest acknowledged packet number if exists, -1 if not */
-static inline int64_t quic_pktns_get_largest_acked_pn(struct quic_pktns *pktns)
-{
-	struct eb64_node *ar = eb64_last(&pktns->rx.arngs.root);
-
-	if (!ar)
-		return -1;
-
-	return eb64_entry(ar, struct quic_arng_node, first)->last;
-}
-
 /* The TX packets sent in the same datagram are linked to each others in
  * the order they are built. This function detach a packet from its successor
  * and predecessor in the same datagram.
@@ -489,50 +454,6 @@ static inline void quic_tx_packet_refdec(struct quic_tx_packet *pkt)
 	}
 }
 
-static inline void quic_pktns_tx_pkts_release(struct quic_pktns *pktns, struct quic_conn *qc)
-{
-	struct eb64_node *node;
-
-	node = eb64_first(&pktns->tx.pkts);
-	while (node) {
-		struct quic_tx_packet *pkt;
-		struct quic_frame *frm, *frmbak;
-
-		pkt = eb64_entry(node, struct quic_tx_packet, pn_node);
-		node = eb64_next(node);
-		if (pkt->flags & QUIC_FL_TX_PACKET_ACK_ELICITING)
-			qc->path->ifae_pkts--;
-		list_for_each_entry_safe(frm, frmbak, &pkt->frms, list) {
-			LIST_DELETE(&frm->list);
-			quic_tx_packet_refdec(frm->pkt);
-			pool_free(pool_head_quic_frame, frm);
-		}
-		eb64_delete(&pkt->pn_node);
-		quic_tx_packet_refdec(pkt);
-	}
-}
-
-/* Discard <pktns> packet number space attached to <qc> QUIC connection.
- * Its loss information are reset. Deduce the outstanding bytes for this
- * packet number space from the outstanding bytes for the path of this
- * connection.
- * Note that all the non acknowledged TX packets and their frames are freed.
- * Always succeeds.
- */
-static inline void quic_pktns_discard(struct quic_pktns *pktns,
-                                      struct quic_conn *qc)
-{
-	qc->path->in_flight -= pktns->tx.in_flight;
-	qc->path->prep_in_flight -= pktns->tx.in_flight;
-	qc->path->loss.pto_count = 0;
-
-	pktns->tx.time_of_last_eliciting = 0;
-	pktns->tx.loss_time = TICK_ETERNITY;
-	pktns->tx.pto_probe = 0;
-	pktns->tx.in_flight = 0;
-	quic_pktns_tx_pkts_release(pktns, qc);
-}
-
 /* Initialize <p> QUIC network path depending on <ipv4> boolean
  * which is true for an IPv4 path, if not false for an IPv6 path.
  */
@@ -575,15 +496,6 @@ static inline size_t quic_path_prep_data(struct quic_path *path)
 	return path->cwnd - path->prep_in_flight;
 }
 
-/* Return 1 if <pktns> matches with the Application packet number space of
- * <conn> connection which is common to the 0-RTT and 1-RTT encryption levels, 0
- * if not (handshake packets).
- */
-static inline int quic_application_pktns(struct quic_pktns *pktns, struct quic_conn *conn)
-{
-	return pktns == &conn->pktns[QUIC_TLS_PKTNS_01RTT];
-}
-
 /* CRYPTO data buffer handling functions. */
 static inline unsigned char *c_buf_getpos(struct quic_enc_level *qel, uint64_t offset)
 {
