@@ -28,8 +28,6 @@ struct tp_preferred_address {
 
 struct tp_version_information {
 	uint32_t choosen;
-	const uint32_t *others;
-	size_t nb_others;
 	const struct quic_version *negotiated_version;
 };
 
